package com.example.demo.task;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 定时任务
 */

@Component
public class DataUpdateTask {

    @Autowired

    private JdbcTemplate masterJdbcTemplate;

    @Autowired
    private JdbcTemplate slaveJdbcTemplate;

    // 每天早上7点执行一次任务
    @Scheduled(cron = "0 0 7 * * ?")
    public void updateData() {

        System.out.println("hello");

        // 获取远程数据库所有表名
        List<String> tableNames = masterJdbcTemplate.queryForList(
                "SELECT table_name FROM information_schema.tables WHERE table_schema = 'tc_zz'",
                String.class);

        // 更新每个表的数据到另一个数据库
        for (String tableName : tableNames) {
            if (!tableName.equals("szs_enterprise_information") && tableName.startsWith("sz") ){
                slaveJdbcTemplate.execute("SET FOREIGN_KEY_CHECKS = 0"); // 禁用外键检查
                slaveJdbcTemplate.execute("delete  FROM tc_zz." + tableName);
                // ... 以此类推，根据实际表的数量进行删除操作
                slaveJdbcTemplate.execute("SET FOREIGN_KEY_CHECKS = 1"); // 启用外键检查
                String selectQuery = "SELECT * FROM tc_zz." + tableName;
                List<Map<String, Object>> tableData = masterJdbcTemplate.queryForList(selectQuery);

                for (Map<String, Object> row : tableData) {
                    StringBuilder insertQuery = new StringBuilder("INSERT INTO tc_zz.")
                            .append(tableName)
                            .append(" (");
                    StringBuilder valuesClause = new StringBuilder(" VALUES (");
                    List<Object> values = new ArrayList<>();

                    for (Map.Entry<String, Object> entry : row.entrySet()) {
                        insertQuery.append(entry.getKey()).append(", ");
                        valuesClause.append("?, ");
                        values.add(entry.getValue());
                    }

                    // 删除最后一个逗号和空格
                    insertQuery.delete(insertQuery.length() - 2, insertQuery.length());
                    valuesClause.delete(valuesClause.length() - 2, valuesClause.length());

                    insertQuery.append(")").append(valuesClause).append(")");
                    slaveJdbcTemplate.update(insertQuery.toString(), values.toArray());
                }
            }
        }

    }



    // 每周日晚上5点执行一次任务
    @Scheduled(cron = "0 0 17 ? * SUN")
    public void updateInformationData() {

        System.out.println("hello");

        // 获取远程数据库所有表名
        List<String> tableNames = masterJdbcTemplate.queryForList(
                "SELECT table_name FROM information_schema.tables WHERE table_schema = 'tc_zz'",
                String.class);

        // 更新每个表的数据到另一个数据库
        for (String tableName : tableNames) {
            if (tableName.equals("szs_enterprise_information") ) {
                // 删除本地数据库所有表的数据
                slaveJdbcTemplate.execute("SET FOREIGN_KEY_CHECKS = 0"); // 禁用外键检查
                slaveJdbcTemplate.execute("delete  FROM tc_zz." + tableName);
                // ... 以此类推，根据实际表的数量进行删除操作
                slaveJdbcTemplate.execute("SET FOREIGN_KEY_CHECKS = 1"); // 启用外键检查
                String selectQuery = "SELECT * FROM tc_zz." + tableName;
                List<Map<String, Object>> tableData = masterJdbcTemplate.queryForList(selectQuery);

                for (Map<String, Object> row : tableData) {
                    StringBuilder insertQuery = new StringBuilder("INSERT INTO tc_zz.")
                            .append(tableName)
                            .append(" (");
                    StringBuilder valuesClause = new StringBuilder(" VALUES (");
                    List<Object> values = new ArrayList<>();

                    for (Map.Entry<String, Object> entry : row.entrySet()) {
                        insertQuery.append(entry.getKey()).append(", ");
                        valuesClause.append("?, ");
                        values.add(entry.getValue());
                    }

                    // 删除最后一个逗号和空格
                    insertQuery.delete(insertQuery.length() - 2, insertQuery.length());
                    valuesClause.delete(valuesClause.length() - 2, valuesClause.length());

                    insertQuery.append(")").append(valuesClause).append(")");
                    slaveJdbcTemplate.update(insertQuery.toString(), values.toArray());
                }
            }
        }

    }
}

